@@ -36,15 +36,17 @@ import pandora.constants as cst
 from pandora.img_tools import compute_std_raster
 
 
-class AbstractDisparity():
+class AbstractDisparity:
     """
     Abstract Disparity class
     """
+
     __metaclass__ = ABCMeta
 
-    disparity_methods_avail = {}
+    disparity_methods_avail: Dict = {}
+    cfg = None
 
-    def __new__(cls, **cfg: dict) -> object:
+    def __new__(cls, **cfg: dict):
         """
         Return the plugin associated with the validation_method given in the configuration
 
@@ -52,24 +54,28 @@ class AbstractDisparity():
         :type cfg: dictionary
         """
         if cls is AbstractDisparity:
-            if isinstance(cfg['disparity_method'], str):
+            if isinstance(cfg["disparity_method"], str):
                 try:
-                    return super(AbstractDisparity, cls).__new__(cls.disparity_methods_avail[cfg['disparity_method']])
+                    return super(AbstractDisparity, cls).__new__(cls.disparity_methods_avail[cfg["disparity_method"]])
                 except KeyError:
-                    logging.error('No disparity method named % supported', cfg['disparity_method'])
+                    logging.error("No disparity method named % supported", cfg["disparity_method"])
                     raise KeyError
             else:
-                if isinstance(cfg['disparity_method'], unicode): # pylint: disable=undefined-variable
+                if isinstance(cfg["disparity_method"], unicode):  # type: ignore # pylint: disable=undefined-variable
                     # creating a plugin from registered short name given as unicode (py2 & 3 compatibility)
                     try:
                         return super(AbstractDisparity, cls).__new__(
-                            cls.disparity_methods_avail[cfg['disparity_method'].encode('utf-8')])
+                            cls.disparity_methods_avail[cfg["disparity_method"].encode("utf-8")]
+                        )
                     except KeyError:
                         logging.error(
-                            'No disparity method named % supported', cfg['disparity_method'])
+                            "No disparity method named % supported",
+                            cfg["disparity_method"],
+                        )
                         raise KeyError
         else:
             return super(AbstractDisparity, cls).__new__(cls)
+        return None
 
     @classmethod
     def register_subclass(cls, short_name: str):
@@ -97,7 +103,7 @@ class AbstractDisparity():
         """
         Describes the disparity method
         """
-        print('Disparity method description')
+        print("Disparity method description")
 
     @abstractmethod
     def to_disp(self, cv: xr.Dataset, img_left: xr.Dataset = None, img_right: xr.Dataset = None) -> xr.Dataset:
@@ -136,20 +142,21 @@ class AbstractDisparity():
         :return: the coefficient map
         :rtype: 2D DataArray (row, col)
         """
-        row = cv.coords['row']
-        col = cv.coords['col']
+        row = cv.coords["row"]
+        col = cv.coords["col"]
 
         # Create the coefficient map
-        coeff_map = xr.DataArray(cv['cost_volume'].sel(disp=cv['disp_indices']).astype(np.float32),
-                                 coords=[('row', row), ('col', col)])
-        coeff_map.name = 'Coefficient Map'
+        coeff_map = xr.DataArray(
+            cv["cost_volume"].sel(disp=cv["disp_indices"]).astype(np.float32),
+            coords=[("row", row), ("col", col)],
+        )
+        coeff_map.name = "Coefficient Map"
         coeff_map.attrs = cv.attrs
 
         return coeff_map
 
     @staticmethod
-    def approximate_right_disparity(cv: xr.Dataset, img_right: xr.Dataset,
-                                    invalid_value: float = 0) -> xr.Dataset:
+    def approximate_right_disparity(cv: xr.Dataset, img_right: xr.Dataset, invalid_value: float = 0) -> xr.Dataset:
         """
         Create the right disparity map, by a diagonal search for the minimum in the left cost volume
 
@@ -177,49 +184,55 @@ class AbstractDisparity():
                 - validity_mask 2D xarray.DataArray (row, col)
         :rtype: xarray.Dataset
         """
-        disp_range = cv.coords['disp'].data.astype(float)
-        col_range = cv.coords['col'].data
-        row_range = cv.coords['row'].data
+        disp_range = cv.coords["disp"].data.astype(float)
+        col_range = cv.coords["col"].data
+        row_range = cv.coords["row"].data
         # Extract integer disparity
         disp_range = np.extract(np.mod(disp_range, 1) == 0, disp_range)
 
         # Allocate the disparity map
         data = np.zeros((len(row_range), len(col_range))).astype(np.float32)
-        disp_map = xr.Dataset({'disparity_map': (['row', 'col'], data)},
-                              coords={'row': cv.coords['row'], 'col': cv.coords['col']})
+        disp_map = xr.Dataset(
+            {"disparity_map": (["row", "col"], data)},
+            coords={"row": cv.coords["row"], "col": cv.coords["col"]},
+        )
 
-        confidence_measure = compute_std_raster(img_right, cv.attrs['window_size'])
+        confidence_measure = compute_std_raster(img_right, cv.attrs["window_size"])
 
         # Create the confidence measure with the original image size and fill it
         confidence_measure_full = np.full((len(row_range), len(col_range), 1), np.nan, dtype=np.float32)
-        offset = cv.attrs['offset_row_col']
+        offset = cv.attrs["offset_row_col"]
         row_off = np.arange(row_range[0] + offset, row_range[-1] - offset + 1)
         col_off = np.arange(col_range[0] + offset, col_range[-1] - offset + 1)
         if offset != 0:
-            confidence_measure_full[offset: - offset, offset: - offset, :] = confidence_measure.reshape(
-                (len(row_off), len(col_off), 1))
+            confidence_measure_full[offset:-offset, offset:-offset, :] = confidence_measure.reshape(
+                (len(row_off), len(col_off), 1)
+            )
         else:
             confidence_measure_full = confidence_measure.reshape((len(row_range), len(col_range), 1))
 
-        disp_map['confidence_measure'] = xr.DataArray(data=confidence_measure_full.astype(np.float32),
-                                                      dims=['row', 'col', 'indicator'])
+        disp_map["confidence_measure"] = xr.DataArray(
+            data=confidence_measure_full.astype(np.float32),
+            dims=["row", "col", "indicator"],
+        )
 
         # Allocate the validity mask
-        disp_map['validity_mask'] = xr.DataArray(np.zeros(disp_map['disparity_map'].shape, dtype=np.uint16),
-                                                 dims=['row', 'col'])
+        disp_map["validity_mask"] = xr.DataArray(
+            np.zeros(disp_map["disparity_map"].shape, dtype=np.uint16),
+            dims=["row", "col"],
+        )
 
         disp_map.attrs = cv.attrs
-        d_range = cv.coords['disp'].data
-        disp_map.attrs['disp_min'] = d_range[0]
-        disp_map.attrs['disp_max'] = d_range[-1]
-        disp_map.attrs['right_left_mode'] = 'approximate'
-        offset = disp_map.attrs['offset_row_col']
+        d_range = cv.coords["disp"].data
+        disp_map.attrs["disp_min"] = d_range[0]
+        disp_map.attrs["disp_max"] = d_range[-1]
+        offset = disp_map.attrs["offset_row_col"]
 
-        indices_nan = np.isnan(cv['cost_volume'].data)
-        if cv.attrs['type_measure'] == 'max':
-            cv['cost_volume'].data[indices_nan] = -np.inf
+        indices_nan = np.isnan(cv["cost_volume"].data)
+        if cv.attrs["type_measure"] == "max":
+            cv["cost_volume"].data[indices_nan] = -np.inf
         else:
-            cv['cost_volume'].data[indices_nan] = np.inf
+            cv["cost_volume"].data[indices_nan] = np.inf
 
         for col in col_range:
             x_d = col - disp_range
@@ -227,43 +240,60 @@ class AbstractDisparity():
 
             # The disparity interval is missing in the left image
             if x_d[valid].size == 0:
-                disp_map['disparity_map'].loc[dict(col=col)] = invalid_value
+                disp_map["disparity_map"].loc[dict(col=col)] = invalid_value
 
                 # Invalid pixel : the disparity interval is missing in the right image
-                disp_map['validity_mask'].loc[
-                    dict(col=col)] += cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING
+                disp_map["validity_mask"].loc[
+                    dict(col=col)
+                ] += cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING
             else:
                 # Diagonal search for the minimum or maximum
 
-                if cv.attrs['type_measure'] == 'max':
-                    min_ = cv['cost_volume'].sel(col=xr.DataArray(np.flip(x_d[valid]), dims='disp_'),
-                                                 disp=xr.DataArray(np.flip(disp_range[valid]),
-                                                                   dims='disp_')).argmax(dim='disp_')
+                if cv.attrs["type_measure"] == "max":
+                    min_ = (
+                        cv["cost_volume"]
+                        .sel(
+                            col=xr.DataArray(np.flip(x_d[valid]), dims="disp_"),
+                            disp=xr.DataArray(np.flip(disp_range[valid]), dims="disp_"),
+                        )
+                        .argmax(dim="disp_")
+                    )
                 else:
-                    min_ = cv['cost_volume'].sel(col=xr.DataArray(np.flip(x_d[valid]), dims='disp_'),
-                                                 disp=xr.DataArray(np.flip(disp_range[valid]),
-                                                                   dims='disp_')).argmin(dim='disp_')
+                    min_ = (
+                        cv["cost_volume"]
+                        .sel(
+                            col=xr.DataArray(np.flip(x_d[valid]), dims="disp_"),
+                            disp=xr.DataArray(np.flip(disp_range[valid]), dims="disp_"),
+                        )
+                        .argmin(dim="disp_")
+                    )
                 # Disparity interval is incomplete
                 if x_d[valid].size != disp_range.size:
                     #  Information: the disparity interval is incomplete (border reached in the right image)
-                    disp_map['validity_mask'].loc[dict(col=col)] += \
-                        cst.PANDORA_MSK_PIXEL_RIGHT_INCOMPLETE_DISPARITY_RANGE
+                    disp_map["validity_mask"].loc[
+                        dict(col=col)
+                    ] += cst.PANDORA_MSK_PIXEL_RIGHT_INCOMPLETE_DISPARITY_RANGE
 
-                disp_map['disparity_map'].loc[dict(col=col)] = -1 * np.flip(disp_range[valid])[min_.data]
+                disp_map["disparity_map"].loc[dict(col=col)] = -1 * np.flip(disp_range[valid])[min_.data]  # type:ignore
 
-        cv['cost_volume'].data[indices_nan] = np.nan
+        cv["cost_volume"].data[indices_nan] = np.nan
         invalid_mc = np.min(indices_nan, axis=2)
         # Pixels where the disparity interval is missing in the right image, have a disparity value invalid_value
         invalid_pixel = np.where(invalid_mc)
-        disp_map['disparity_map'].data[invalid_pixel] = invalid_value
+        disp_map["disparity_map"].data[invalid_pixel] = invalid_value
 
         if offset > 0:
             AbstractDisparity.mask_border(disp_map)
 
         return disp_map
 
-    def validity_mask(self, disp: xr.Dataset, img_left: xr.Dataset, img_right: xr.Dataset,
-                      cv: xr.Dataset) -> None:
+    def validity_mask(
+        self,
+        disp: xr.Dataset,
+        img_left: xr.Dataset,
+        img_right: xr.Dataset,
+        cv: xr.Dataset,
+    ) -> None:
         """
         Create the validity mask of the disparity map
 
@@ -290,43 +320,47 @@ class AbstractDisparity():
         :return: None
         """
         # Allocate the validity mask
-        disp['validity_mask'] = xr.DataArray(np.zeros(disp['disparity_map'].shape, dtype=np.uint16),
-                                             dims=['row', 'col'])
+        disp["validity_mask"] = xr.DataArray(
+            np.zeros(disp["disparity_map"].shape, dtype=np.uint16), dims=["row", "col"]
+        )
 
-        d_min = int(disp.attrs['disp_min'])
-        d_max = int(disp.attrs['disp_max'])
-        col = disp.coords['col'].data
+        d_min = int(disp.attrs["disp_min"])
+        d_max = int(disp.attrs["disp_max"])
+        col = disp.coords["col"].data
 
         # Since disparity map is full size (input images size)
-        offset = disp.attrs['offset_row_col']
+        offset = disp.attrs["offset_row_col"]
 
         # Negative disparity range
         if d_max < 0:
-            bit_1 = np.where((col + d_max) < (col[0]+offset))
+            bit_1 = np.where((col + d_max) < (col[0] + offset))
             # Information: the disparity interval is incomplete (border reached in the right image)
-            disp['validity_mask'].data[:, np.where(((col + d_max) >= (col[0]+offset)) &
-                                                   ((col + d_min) < (col[0]+offset)))] \
-                +=cst.PANDORA_MSK_PIXEL_RIGHT_INCOMPLETE_DISPARITY_RANGE
+            disp["validity_mask"].data[
+                :,
+                np.where(((col + d_max) >= (col[0] + offset)) & ((col + d_min) < (col[0] + offset))),
+            ] += cst.PANDORA_MSK_PIXEL_RIGHT_INCOMPLETE_DISPARITY_RANGE
         else:
             # Positive disparity range
             if d_min > 0:
-                bit_1 = np.where((col + d_min) > (col[-1]-offset))
+                bit_1 = np.where((col + d_min) > (col[-1] - offset))
                 # Information: the disparity interval is incomplete (border reached in the right image)
-                disp['validity_mask'].data[:, np.where(((col + d_min) <= (col[-1]-offset)) &
-                                                       ((col + d_max) > (col[-1]-offset)))] += \
-                    cst.PANDORA_MSK_PIXEL_RIGHT_INCOMPLETE_DISPARITY_RANGE
+                disp["validity_mask"].data[
+                    :,
+                    np.where(((col + d_min) <= (col[-1] - offset)) & ((col + d_max) > (col[-1] - offset))),
+                ] += cst.PANDORA_MSK_PIXEL_RIGHT_INCOMPLETE_DISPARITY_RANGE
 
             # Disparity range contains 0
             else:
                 bit_1 = ([],)
                 # Information: the disparity interval is incomplete (border reached in the right image)
-                disp['validity_mask'].data[:, np.where(((col + d_min) < (col[0]+offset)) |
-                                                       (col + d_max > (col[-1])-offset))] += \
-                    cst.PANDORA_MSK_PIXEL_RIGHT_INCOMPLETE_DISPARITY_RANGE
+                disp["validity_mask"].data[
+                    :,
+                    np.where(((col + d_min) < (col[0] + offset)) | (col + d_max > (col[-1]) - offset)),
+                ] += cst.PANDORA_MSK_PIXEL_RIGHT_INCOMPLETE_DISPARITY_RANGE
 
         # Invalid pixel : the disparity interval is missing in the right image ( disparity range
         # outside the image )
-        disp['validity_mask'].data[:, bit_1] += cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING
+        disp["validity_mask"].data[:, bit_1] += cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING
 
         # The disp_min and disp_max used to search missing disparity interval are not the local disp_min and disp_max
         # in case of a variable range of disparities. So there may be pixels that have missing disparity range (all
@@ -335,10 +369,10 @@ class AbstractDisparity():
 
         self.mask_invalid_variable_disparity_range(disp, cv)
 
-        if 'msk' in img_left.data_vars:
+        if "msk" in img_left.data_vars:
             self.allocate_left_mask(disp, img_left)
 
-        if 'msk' in img_right.data_vars:
+        if "msk" in img_right.data_vars:
             self.allocate_right_mask(disp, img_right, bit_1)
         if offset > 0:
             self.mask_border(disp)
@@ -355,15 +389,14 @@ class AbstractDisparity():
         :type disp: xarray.Dataset
         :return: None
         """
-        offset = disp.attrs['offset_row_col']
+        offset = disp.attrs["offset_row_col"]
 
         if offset > 0:
             # Border pixels have invalid disparity, erase the potential previous values
-            disp['validity_mask'].data[:offset, :] = cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
-            disp['validity_mask'].data[-offset:, :] = cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
-            disp['validity_mask'].data[offset:-offset, :offset] = cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
-            disp['validity_mask'].data[offset:-offset, -offset:] = cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
-
+            disp["validity_mask"].data[:offset, :] = cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
+            disp["validity_mask"].data[-offset:, :] = cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
+            disp["validity_mask"].data[offset:-offset, :offset] = cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
+            disp["validity_mask"].data[offset:-offset, -offset:] = cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
 
     @staticmethod
     def mask_invalid_variable_disparity_range(disp, cv) -> None:
@@ -383,20 +416,24 @@ class AbstractDisparity():
         :type cv: xarray.Dataset
         :return: None
         """
-        indices_nan = np.isnan(cv['cost_volume'].data)
+        indices_nan = np.isnan(cv["cost_volume"].data)
         missing_disparity_range = np.min(indices_nan, axis=2)
         missing_range_y, missing_range_x = np.where(missing_disparity_range)
 
         # Mask the positions which have an missing disparity range, not already taken into account
-        condition_to_mask = (disp['validity_mask'].data[missing_range_y, missing_range_x] &
-                             cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING) == 0
-        masking_value = disp['validity_mask'].data[missing_range_y, missing_range_x] + \
-                        cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING
-        no_masking_value = disp['validity_mask'].data[missing_range_y, missing_range_x]
+        condition_to_mask = (
+            disp["validity_mask"].data[missing_range_y, missing_range_x]
+            & cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING
+        ) == 0
+        masking_value = (
+            disp["validity_mask"].data[missing_range_y, missing_range_x]
+            + cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING
+        )
+        no_masking_value = disp["validity_mask"].data[missing_range_y, missing_range_x]
 
-        disp['validity_mask'].data[missing_range_y, missing_range_x] = np.where(condition_to_mask,
-                                                                                masking_value,
-                                                                                no_masking_value)
+        disp["validity_mask"].data[missing_range_y, missing_range_x] = np.where(
+            condition_to_mask, masking_value, no_masking_value
+        )
 
     @staticmethod
     def allocate_left_mask(disp: xr.Dataset, img_left: xr.Dataset) -> None:
@@ -415,20 +452,24 @@ class AbstractDisparity():
         :type img_left: xarray.Dataset
         :return: None
         """
-        _, r_mask = xr.align(disp['validity_mask'], img_left['msk'])
+        _, r_mask = xr.align(disp["validity_mask"], img_left["msk"])
 
         # Dilatation : pixels that contains no_data in their aggregation window become no_data
-        dil = binary_dilation(img_left['msk'].data == img_left.attrs['no_data_mask'],
-                              structure=np.ones((disp.attrs['window_size'], disp.attrs['window_size'])),
-                              iterations=1)
+        dil = binary_dilation(
+            img_left["msk"].data == img_left.attrs["no_data_mask"],
+            structure=np.ones((disp.attrs["window_size"], disp.attrs["window_size"])),
+            iterations=1,
+        )
 
         # Invalid pixel : no_data in the left image
-        disp['validity_mask'] += dil.astype(np.uint16) * cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
+        disp["validity_mask"] += dil.astype(np.uint16) * cst.PANDORA_MSK_PIXEL_LEFT_NODATA_OR_BORDER
 
         # Invalid pixel : invalidated by the validity mask of the left image given as input
-        disp['validity_mask'] += xr.where(
-            (r_mask != img_left.attrs['no_data_mask']) & (r_mask != img_left.attrs['valid_pixels']),
-            cst.PANDORA_MSK_PIXEL_IN_VALIDITY_MASK_LEFT, 0).astype(np.uint16)
+        disp["validity_mask"] += xr.where(
+            (r_mask != img_left.attrs["no_data_mask"]) & (r_mask != img_left.attrs["valid_pixels"]),
+            cst.PANDORA_MSK_PIXEL_IN_VALIDITY_MASK_LEFT,
+            0,
+        ).astype(np.uint16)
 
     @staticmethod
     def allocate_right_mask(disp: xr.Dataset, img_right: xr.Dataset, bit_1: Union[np.ndarray, Tuple]) -> None:
@@ -450,21 +491,26 @@ class AbstractDisparity():
         :return: None
         """
 
-        offset = disp.attrs['offset_row_col']
+        offset = disp.attrs["offset_row_col"]
 
-        _, r_mask = xr.align(disp['validity_mask'], img_right['msk'])
-        d_min = int(disp.attrs['disp_min'])
-        d_max = int(disp.attrs['disp_max'])
-        col = disp.coords['col'].data
-        row = disp.coords['row'].data
+        _, r_mask = xr.align(disp["validity_mask"], img_right["msk"])
+        d_min = int(disp.attrs["disp_min"])
+        d_max = int(disp.attrs["disp_max"])
+        col = disp.coords["col"].data
+        row = disp.coords["row"].data
 
         # Dilatation : pixels that contains no_data in their aggregation window become no_data
-        dil = binary_dilation(img_right['msk'].data == img_right.attrs['no_data_mask'],
-                              structure=np.ones((disp.attrs['window_size'], disp.attrs['window_size'])),
-                              iterations=1)
+        dil = binary_dilation(
+            img_right["msk"].data == img_right.attrs["no_data_mask"],
+            structure=np.ones((disp.attrs["window_size"], disp.attrs["window_size"])),
+            iterations=1,
+        )
 
-        r_mask = xr.where((r_mask != img_right.attrs['no_data_mask']) & (r_mask != img_right.attrs['valid_pixels']),
-                          1, 0).data
+        r_mask = xr.where(
+            (r_mask != img_right.attrs["no_data_mask"]) & (r_mask != img_right.attrs["valid_pixels"]),
+            1,
+            0,
+        ).data
 
         # Useful to calculate the case where the disparity interval is incomplete, and all remaining right
         # positions are invalidated by the right mask
@@ -476,7 +522,7 @@ class AbstractDisparity():
         for dsp in range(d_min, d_max + 1):
             # Diagonal in the cost volume
             col_d = col_range + dsp
-            valid_index = np.where((col_d >= col_range[0]+offset) & (col_d <= col_range[-1]-offset))
+            valid_index = np.where((col_d >= col_range[0] + offset) & (col_d <= col_range[-1] - offset))
 
             # No_data and masked pixels do not raise the same flag, we need to treat them differently
             b_2_7[:, col_range[valid_index]] += r_mask[:, col_d[valid_index]].astype(np.uint16)
@@ -490,20 +536,23 @@ class AbstractDisparity():
             no_data_right[:, bit_1[0]] = 0
 
             # Invalid pixel: right positions invalidated by the mask of the right image given as input
-            disp['validity_mask'].data[np.where(b_2_7 == len(range(d_min, d_max + 1)))] += \
-                cst.PANDORA_MSK_PIXEL_IN_VALIDITY_MASK_RIGHT
+            disp["validity_mask"].data[
+                np.where(b_2_7 == len(range(d_min, d_max + 1)))
+            ] += cst.PANDORA_MSK_PIXEL_IN_VALIDITY_MASK_RIGHT
 
             # If Invalid pixel : the disparity interval is missing in the right image (disparity interval
             # is invalidated by no_data in the right image )
-            disp['validity_mask'].data[np.where(no_data_right == len(range(d_min, d_max + 1)))] += \
-                cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING
+            disp["validity_mask"].data[
+                np.where(no_data_right == len(range(d_min, d_max + 1)))
+            ] += cst.PANDORA_MSK_PIXEL_RIGHT_NODATA_OR_DISPARITY_RANGE_MISSING
 
 
-@AbstractDisparity.register_subclass('wta')
+@AbstractDisparity.register_subclass("wta")
 class WinnerTakesAll(AbstractDisparity):
     """
     WinnerTakesAll class allows to perform the disparity computation step
     """
+
     # Default configuration, do not change this value
     _INVALID_DISPARITY = -9999
 
@@ -513,7 +562,7 @@ class WinnerTakesAll(AbstractDisparity):
         :type cfg: dictionary
         """
         self.cfg = self.check_conf(**cfg)
-        self._invalid_disparity = self.cfg['invalid_disparity']
+        self._invalid_disparity = self.cfg["invalid_disparity"]
 
     def check_conf(self, **cfg: Union[str, int, float, bool]) -> Dict[str, Union[str, int, float, bool]]:
         """
@@ -525,12 +574,12 @@ class WinnerTakesAll(AbstractDisparity):
         :rtype: dict
         """
         # Give the default value if the required element is not in the configuration
-        if 'invalid_disparity' not in cfg:
-            cfg['invalid_disparity'] = self._INVALID_DISPARITY
+        if "invalid_disparity" not in cfg:
+            cfg["invalid_disparity"] = self._INVALID_DISPARITY
 
         schema = {
-            'disparity_method': And(str, lambda input: 'wta'),
-            'invalid_disparity': Or(int, float, lambda input: np.isnan(input))
+            "disparity_method": And(str, lambda input: "wta"),
+            "invalid_disparity": Or(int, float, lambda input: np.isnan(input)),
         }
 
         checker = Checker(schema)
@@ -542,7 +591,7 @@ class WinnerTakesAll(AbstractDisparity):
         Describes the disparity method
         :return: None
         """
-        print('Winner takes all method')
+        print("Winner takes all method")
 
     def to_disp(self, cv: xr.Dataset, img_left: xr.Dataset = None, img_right: xr.Dataset = None) -> xr.Dataset:
         """
@@ -569,40 +618,41 @@ class WinnerTakesAll(AbstractDisparity):
                 - confidence_measure 3D xarray.DataArray (row, col, indicator)
         :rtype: xarray.Dataset
         """
-        indices_nan = np.isnan(cv['cost_volume'].data)
+        indices_nan = np.isnan(cv["cost_volume"].data)
 
         # Winner Takes All strategy
-        if cv.attrs['type_measure'] == 'max':
+        if cv.attrs["type_measure"] == "max":
             # Disparities are computed by selecting the maximal cost at each pixel
-            cv['cost_volume'].data[indices_nan] = -np.inf
+            cv["cost_volume"].data[indices_nan] = -np.inf
             disp = self.argmax_split(cv)
         else:
             # Disparities are computed by selecting the minimal cost at each pixel
-            cv['cost_volume'].data[indices_nan] = np.inf
+            cv["cost_volume"].data[indices_nan] = np.inf
             disp = self.argmin_split(cv)
 
-        cv['cost_volume'].data[indices_nan] = np.nan
-        row = cv.coords['row']
-        col = cv.coords['col']
+        cv["cost_volume"].data[indices_nan] = np.nan
+        row = cv.coords["row"]
+        col = cv.coords["col"]
 
         # ----- Disparity map -----
-        disp_map = xr.Dataset({'disparity_map': (['row', 'col'], disp)}, coords={'row': row, 'col': col})
+        disp_map = xr.Dataset({"disparity_map": (["row", "col"], disp)}, coords={"row": row, "col": col})
         invalid_mc = np.min(indices_nan, axis=2)
         # Pixels where the disparity interval is missing in the right image, have a disparity value invalid_value
         invalid_pixel = np.where(invalid_mc)
-        disp_map['disparity_map'].data[invalid_pixel] = self._invalid_disparity
+        disp_map["disparity_map"].data[invalid_pixel] = self._invalid_disparity
 
         # Save the disparity map in the cost volume
-        cv['disp_indices'] = disp_map['disparity_map'].copy(deep=True)
+        cv["disp_indices"] = disp_map["disparity_map"].copy(deep=True)
 
         disp_map.attrs = cv.attrs
-        d_range = cv.coords['disp'].data
-        disp_map.attrs['disp_min'] = d_range[0]
-        disp_map.attrs['disp_max'] = d_range[-1]
+        d_range = cv.coords["disp"].data
+        disp_map.attrs["disp_min"] = d_range[0]
+        disp_map.attrs["disp_max"] = d_range[-1]
 
         # ----- Confidence measure -----
         # Allocate the confidence measure in the disparity_map dataset
-        disp_map['confidence_measure'] = cv['confidence_measure']
+        if "confidence_measure" in cv.data_vars:
+            disp_map["confidence_measure"] = cv["confidence_measure"]
 
         # Remove temporary values
         del indices_nan
@@ -621,24 +671,25 @@ class WinnerTakesAll(AbstractDisparity):
         :return: the disparities for which the cost volume values are the smallest
         :rtype: np.ndarray
         """
-        ncol, nrow, ndsp = cost_volume['cost_volume'].shape # pylint: disable=unused-variable
+        ncol, nrow, ndsp = cost_volume["cost_volume"].shape  # pylint: disable=unused-variable
         disp = np.zeros((ncol, nrow), dtype=np.float32)
 
         # Numpy argmin is making a copy of the cost volume.
         # To reduce memory, numpy argmin is applied on a small part of the cost volume.
         # The cost volume is split (along the row axis) into multiple sub-arrays with a step of 100
-        cv_chunked_y = np.array_split(cost_volume['cost_volume'].data, np.arange(100, ncol, 100), axis=0)
+        cv_chunked_y = np.array_split(cost_volume["cost_volume"].data, np.arange(100, ncol, 100), axis=0)
 
         y_begin = 0
 
-        for col, cv_y in enumerate(cv_chunked_y): # pylint: disable=unused-variable
+        for col, cv_y in enumerate(cv_chunked_y):  # pylint: disable=unused-variable
             # To reduce memory, the cost volume is split (along the col axis) into
             # multiple sub-arrays with a step of 100
             cv_chunked_x = np.array_split(cv_y, np.arange(100, nrow, 100), axis=1)
             x_begin = 0
-            for row, cv_x in enumerate(cv_chunked_x): # pylint: disable=unused-variable
-                disp[y_begin:y_begin + cv_y.shape[0], x_begin: x_begin + cv_x.shape[1]] = \
-                    cost_volume.coords['disp'].data[np.argmin(cv_x, axis=2)]
+            for row, cv_x in enumerate(cv_chunked_x):  # pylint: disable=unused-variable
+                disp[y_begin : y_begin + cv_y.shape[0], x_begin : x_begin + cv_x.shape[1]] = cost_volume.coords[
+                    "disp"
+                ].data[np.argmin(cv_x, axis=2)]
                 x_begin += cv_x.shape[1]
 
             y_begin += cv_y.shape[0]
@@ -656,25 +707,26 @@ class WinnerTakesAll(AbstractDisparity):
         :return: the disparities for which the cost volume values are the highest
         :rtype: np.ndarray
         """
-        ncol, nrow, ndisp = cost_volume['cost_volume'].shape # pylint: disable=unused-variable
+        ncol, nrow, ndisp = cost_volume["cost_volume"].shape  # pylint: disable=unused-variable
         disp = np.zeros((ncol, nrow), dtype=np.float32)
 
         # Numpy argmax is making a copy of the cost volume.
         # To reduce memory, numpy argmax is applied on a small part of the cost volume.
         # The cost volume is split (along the row axis) into multiple sub-arrays with a step of 100
-        cv_chunked_col = np.array_split(cost_volume['cost_volume'].data, np.arange(100, ncol, 100), axis=0)
+        cv_chunked_col = np.array_split(cost_volume["cost_volume"].data, np.arange(100, ncol, 100), axis=0)
 
         col_begin = 0
 
-        for col, cv_y in enumerate(cv_chunked_col): # pylint: disable=unused-variable
+        for col, cv_y in enumerate(cv_chunked_col):  # pylint: disable=unused-variable
             # To reduce memory, the cost volume is split (along the col axis)
             # into multiple sub-arrays with a step of 100
             cv_chunked_row = np.array_split(cv_y, np.arange(100, nrow, 100), axis=1)
             row_begin = 0
-            for row, cv_x in enumerate(cv_chunked_row): # pylint: disable=unused-variable
-                disp[col_begin:col_begin + cv_y.shape[0],
-                row_begin: row_begin + cv_x.shape[1]] = \
-                    cost_volume.coords['disp'].data[np.argmax(cv_x, axis=2)]
+            for row, cv_x in enumerate(cv_chunked_row):  # pylint: disable=unused-variable
+                disp[
+                    col_begin : col_begin + cv_y.shape[0],
+                    row_begin : row_begin + cv_x.shape[1],
+                ] = cost_volume.coords["disp"].data[np.argmax(cv_x, axis=2)]
                 row_begin += cv_x.shape[1]
 
             col_begin += cv_y.shape[0]
