@@ -31,7 +31,7 @@ import xarray as xr
 from json_checker import Checker, And
 
 try:
-    import graphviz # pylint: disable=unused-import
+    import graphviz  # pylint: disable=unused-import
     from transitions.extensions import GraphMachine as Machine
 except ImportError:
     from transitions import Machine
@@ -45,49 +45,147 @@ from pandora import optimization
 from pandora import refinement
 from pandora import matching_cost
 from pandora import validation
+from pandora import cost_volume_confidence
 from .img_tools import prepare_pyramid
 
 
-
 # This module contains class associated to the pandora state machine
 
+
 class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
     """
     PandoraMachine class to create and use a state machine
     """
 
     _transitions_run = [
-        {'trigger': 'matching_cost', 'source': 'begin', 'dest': 'cost_volume', 'after': 'matching_cost_run'},
-        {'trigger': 'aggregation', 'source': 'cost_volume', 'dest': 'cost_volume', 'after': 'aggregation_run'},
-        {'trigger': 'optimization', 'source': 'cost_volume', 'dest': 'cost_volume', 'after': 'optimization_run'},
-        {'trigger': 'disparity', 'source': 'cost_volume', 'dest': 'disp_map', 'after': 'disparity_run'},
-        {'trigger': 'filter', 'source': 'disp_map', 'dest': 'disp_map', 'after': 'filter_run'},
-        {'trigger': 'refinement', 'source': 'disp_map', 'dest': 'disp_map', 'after': 'refinement_run'},
-        {'trigger': 'validation', 'source': 'disp_map', 'dest': 'disp_map', 'after': 'validation_run'},
+        {
+            "trigger": "matching_cost",
+            "source": "begin",
+            "dest": "cost_volume",
+            "after": "matching_cost_run",
+        },
+        {
+            "trigger": "aggregation",
+            "source": "cost_volume",
+            "dest": "cost_volume",
+            "after": "aggregation_run",
+        },
+        {
+            "trigger": "optimization",
+            "source": "cost_volume",
+            "dest": "cost_volume",
+            "after": "optimization_run",
+        },
+        {
+            "trigger": "disparity",
+            "source": "cost_volume",
+            "dest": "disp_map",
+            "after": "disparity_run",
+        },
+        {
+            "trigger": "filter",
+            "source": "disp_map",
+            "dest": "disp_map",
+            "after": "filter_run",
+        },
+        {
+            "trigger": "refinement",
+            "source": "disp_map",
+            "dest": "disp_map",
+            "after": "refinement_run",
+        },
+        {
+            "trigger": "validation",
+            "source": "disp_map",
+            "dest": "disp_map",
+            "after": "validation_run",
+        },
         # Conditional state change, if it is the last scale the multiscale state will not be triggered
         # This way, after the last scale we can still apply a filter state
-        {'trigger': 'multiscale', 'source': 'disp_map', 'conditions': 'is_not_last_scale',
-         'dest': 'begin', 'after': 'run_multiscale'},
+        {
+            "trigger": "multiscale",
+            "source": "disp_map",
+            "conditions": "is_not_last_scale",
+            "dest": "begin",
+            "after": "run_multiscale",
+        },
+        {
+            "trigger": "cost_volume_confidence",
+            "source": "cost_volume",
+            "dest": "cost_volume",
+            "after": "cost_volume_confidence_run",
+        },
     ]
 
     _transitions_check = [
-        {'trigger': 'matching_cost', 'source': 'begin', 'dest': 'cost_volume', 'before': 'right_disp_map_check_conf',
-         'after': 'matching_cost_check_conf'},
-        {'trigger': 'aggregation', 'source': 'cost_volume', 'dest': 'cost_volume', 'after': 'aggregation_check_conf'},
-        {'trigger': 'optimization', 'source': 'cost_volume', 'dest': 'cost_volume', 'after': 'optimization_check_conf'},
-        {'trigger': 'disparity', 'source': 'cost_volume', 'dest': 'disp_map', 'after': 'disparity_check_conf'},
-        {'trigger': 'filter', 'source': 'disp_map', 'dest': 'disp_map', 'after': 'filter_check_conf'},
-        {'trigger': 'refinement', 'source': 'disp_map', 'dest': 'disp_map', 'after': 'refinement_check_conf'},
-        {'trigger': 'validation', 'source': 'disp_map', 'dest': 'disp_map', 'after': 'validation_check_conf'},
+        {
+            "trigger": "matching_cost",
+            "source": "begin",
+            "dest": "cost_volume",
+            "before": "right_disp_map_check_conf",
+            "after": "matching_cost_check_conf",
+        },
+        {
+            "trigger": "aggregation",
+            "source": "cost_volume",
+            "dest": "cost_volume",
+            "after": "aggregation_check_conf",
+        },
+        {
+            "trigger": "optimization",
+            "source": "cost_volume",
+            "dest": "cost_volume",
+            "after": "optimization_check_conf",
+        },
+        {
+            "trigger": "disparity",
+            "source": "cost_volume",
+            "dest": "disp_map",
+            "after": "disparity_check_conf",
+        },
+        {
+            "trigger": "filter",
+            "source": "disp_map",
+            "dest": "disp_map",
+            "after": "filter_check_conf",
+        },
+        {
+            "trigger": "refinement",
+            "source": "disp_map",
+            "dest": "disp_map",
+            "after": "refinement_check_conf",
+        },
+        {
+            "trigger": "validation",
+            "source": "disp_map",
+            "dest": "disp_map",
+            "after": "validation_check_conf",
+        },
         # For the check conf we define the destination of multiscale state as disp_map instead of begin
         # given the conditional change of state
-        {'trigger': 'multiscale', 'source': 'disp_map', 'dest': 'disp_map', 'after': 'multiscale_check_conf'}
+        {
+            "trigger": "multiscale",
+            "source": "disp_map",
+            "dest": "disp_map",
+            "after": "multiscale_check_conf",
+        },
+        {
+            "trigger": "cost_volume_confidence",
+            "source": "cost_volume",
+            "dest": "cost_volume",
+            "after": "cost_volume_confidence_check_conf",
+        },
     ]
 
-    def __init__(self, img_left_pyramid: List[xr.Dataset] = None, img_right_pyramid: List[xr.Dataset] = None,
-                 disp_min: Union[np.array, int] = None, disp_max: Union[np.array, int] = None,
-                 right_disp_min: Union[np.array, None] = None, right_disp_max: Union[np.array, None] = None
-                 ) -> None:
+    def __init__(
+        self,
+        img_left_pyramid: List[xr.Dataset] = None,
+        img_right_pyramid: List[xr.Dataset] = None,
+        disp_min: Union[np.array, int] = None,
+        disp_max: Union[np.array, int] = None,
+        right_disp_min: Union[np.array, None] = None,
+        right_disp_max: Union[np.array, None] = None,
+    ) -> None:
 
         """
         Initialize Pandora Machine
@@ -113,58 +211,64 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :return: None
         """
         # Left image scale pyramid
-        self.img_left_pyramid = img_left_pyramid
+        self.img_left_pyramid: List[xr.Dataset] = img_left_pyramid
         # Right image scale pyramid
-        self.img_right_pyramid = img_right_pyramid
+        self.img_right_pyramid: List[xr.Dataset] = img_right_pyramid
         # Left image
-        self.left_img = None
+        self.left_img: xr.Dataset = None
         # Right image
-        self.right_img = None
+        self.right_img: xr.Dataset = None
         # Minimum disparity
-        self.disp_min = disp_min
+        self.disp_min: Union[np.array, int] = disp_min
         # Maximum disparity
-        self.disp_max = disp_max
+        self.disp_max: Union[np.array, int] = disp_max
         # Maximum disparity for the right image
-        self.right_disp_min = right_disp_min
+        self.right_disp_min: Union[np.array, None] = right_disp_min
         # Minimum disparity for the right image
-        self.right_disp_max = right_disp_max
+        self.right_disp_max: Union[np.array, None] = right_disp_max
         # User minimum disparity
-        self.dmin_user = None
+        self.dmin_user: Union[np.array, int] = None
         # User maximum disparity
-        self.dmax_user = None
+        self.dmax_user: Union[np.array, int] = None
         # User minimum disparity right
-        self.dmin_user_right = None
+        self.dmin_user_right: Union[np.array, None] = None
         # User maximum disparity right
-        self.dmax_user_right = None
+        self.dmax_user_right: Union[np.array, None] = None
 
         # Scale factor
-        self.scale_factor = None
+        self.scale_factor: int = None
         # Number of scales
-        self.num_scales = 1
+        self.num_scales: int = 1
         # Current scale
-        self.current_scale = None
+        self.current_scale: int = None
 
         # left cost volume
-        self.left_cv = None
+        self.left_cv: xr.Dataset = None
         # right cost volume
-        self.right_cv = None
+        self.right_cv: xr.Dataset = None
         # left disparity map
-        self.left_disparity = None
+        self.left_disparity: xr.Dataset = None
         # right disparity map
-        self.right_disparity = None
+        self.right_disparity: xr.Dataset = None
 
         # Pandora's pipeline configuration
-        self.pipeline_cfg = {'pipeline': {}}
+        self.pipeline_cfg: Dict = {"pipeline": {}}
         # Right disparity map computation information: Can be "none" or "accurate"
         # Useful during the running steps to choose if we must compute left and right objects.
-        self.right_disp_map = 'none'
+        self.right_disp_map = "none"
         # Define avalaible states
-        states_ = ['begin', 'cost_volume', 'disp_map']
+        states_ = ["begin", "cost_volume", "disp_map"]
 
         # Initialize a machine without any transition
-        Machine.__init__(self, states=states_, initial='begin', transitions=None, auto_transitions=False)
+        Machine.__init__(
+            self,
+            states=states_,
+            initial="begin",
+            transitions=None,
+            auto_transitions=False,
+        )
 
-        logging.getLogger('transitions').setLevel(logging.WARNING)
+        logging.getLogger("transitions").setLevel(logging.WARNING)
 
     def matching_cost_run(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -176,8 +280,8 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :return: None
         """
 
-        logging.info('Matching cost computation...')
-        matching_cost_ = matching_cost.AbstractMatchingCost(**cfg[input_step])
+        logging.info("Matching cost computation...")
+        matching_cost_ = matching_cost.AbstractMatchingCost(**cfg[input_step])  # type: ignore
 
         # Update min and max disparity according to the current scale
         self.disp_min = self.disp_min * self.scale_factor
@@ -186,13 +290,25 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         dmin_min, dmax_max = matching_cost_.dmin_dmax(self.disp_min, self.disp_max)
 
         # Compute cost volume and mask it
-        if self.right_disp_map != 'accurate':
+        if self.right_disp_map != "accurate":
             self.left_cv = matching_cost_.compute_cost_volume(self.left_img, self.right_img, dmin_min, dmax_max)
-            matching_cost_.cv_masked(self.left_img, self.right_img, self.left_cv, self.disp_min, self.disp_max)
+            matching_cost_.cv_masked(
+                self.left_img,
+                self.right_img,
+                self.left_cv,
+                self.disp_min,
+                self.disp_max,
+            )
 
         else:
             self.left_cv = matching_cost_.compute_cost_volume(self.left_img, self.right_img, dmin_min, dmax_max)
-            matching_cost_.cv_masked(self.left_img, self.right_img, self.left_cv, self.disp_min, self.disp_max)
+            matching_cost_.cv_masked(
+                self.left_img,
+                self.right_img,
+                self.left_cv,
+                self.disp_min,
+                self.disp_max,
+            )
 
             # Update min and max disparity according to the current scale
             self.right_disp_min = self.right_disp_min * self.scale_factor
@@ -200,10 +316,16 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
             # Obtain absolute min and max right disparities
             dmin_min_right, dmax_max_right = matching_cost_.dmin_dmax(self.right_disp_min, self.right_disp_max)
             # Compute right cost volume and mask it
-            self.right_cv = matching_cost_.compute_cost_volume(self.right_img, self.left_img, dmin_min_right,
-                                                               dmax_max_right)
-            matching_cost_.cv_masked(self.right_img, self.left_img, self.right_cv, self.right_disp_min,
-                              self.right_disp_max)
+            self.right_cv = matching_cost_.compute_cost_volume(
+                self.right_img, self.left_img, dmin_min_right, dmax_max_right
+            )
+            matching_cost_.cv_masked(
+                self.right_img,
+                self.left_img,
+                self.right_cv,
+                self.right_disp_min,
+                self.right_disp_max,
+            )
 
     def aggregation_run(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -214,8 +336,8 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: str
         :return: None
         """
-        aggregation_ = aggregation.AbstractAggregation(**cfg[input_step])
-        if self.right_disp_map != 'accurate':
+        aggregation_ = aggregation.AbstractAggregation(**cfg[input_step])  # type: ignore
+        if self.right_disp_map != "accurate":
             aggregation_.cost_volume_aggregation(self.left_img, self.right_img, self.left_cv)
         else:
             aggregation_.cost_volume_aggregation(self.left_img, self.right_img, self.left_cv)
@@ -230,9 +352,9 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: str
         :return: None
         """
-        optimization_ = optimization.AbstractOptimization(**cfg[input_step])
-        logging.info('Cost optimization...')
-        if self.right_disp_map != 'accurate':
+        optimization_ = optimization.AbstractOptimization(**cfg[input_step])  # type: ignore
+        logging.info("Cost optimization...")
+        if self.right_disp_map != "accurate":
             self.left_cv = optimization_.optimize_cv(self.left_cv, self.left_img, self.right_img)
         else:
             self.left_cv = optimization_.optimize_cv(self.left_cv, self.left_img, self.right_img)
@@ -247,19 +369,16 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: str
         :return: None
         """
-        logging.info('Disparity computation...')
-        disparity_ = disparity.AbstractDisparity(**cfg[input_step])
-        if self.right_disp_map != 'accurate':
+        logging.info("Disparity computation...")
+        disparity_ = disparity.AbstractDisparity(**cfg[input_step])  # type: ignore
+        if self.right_disp_map != "accurate":
             self.left_disparity = disparity_.to_disp(self.left_cv, self.left_img, self.right_img)
-            disparity_.validity_mask(self.left_disparity, self.left_img, self.right_img,
-                                     self.left_cv)
-        elif self.right_disp_map == 'accurate':
+            disparity_.validity_mask(self.left_disparity, self.left_img, self.right_img, self.left_cv)
+        elif self.right_disp_map == "accurate":
             self.left_disparity = disparity_.to_disp(self.left_cv, self.left_img, self.right_img)
-            disparity_.validity_mask(self.left_disparity, self.left_img, self.right_img,
-                                     self.left_cv)
+            disparity_.validity_mask(self.left_disparity, self.left_img, self.right_img, self.left_cv)
             self.right_disparity = disparity_.to_disp(self.right_cv, self.right_img, self.left_img)
-            disparity_.validity_mask(self.right_disparity, self.right_img, self.left_img,
-                                     self.right_cv)
+            disparity_.validity_mask(self.right_disparity, self.right_img, self.left_img, self.right_cv)
 
     def filter_run(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -270,9 +389,9 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: str
         :return: None
         """
-        logging.info('Disparity filtering...')
-        filter_ = filter.AbstractFilter(**cfg[input_step])
-        if self.right_disp_map != 'accurate':
+        logging.info("Disparity filtering...")
+        filter_ = filter.AbstractFilter(**cfg[input_step])  # type: ignore
+        if self.right_disp_map != "accurate":
             filter_.filter_disparity(self.left_disparity)
         else:
             filter_.filter_disparity(self.left_disparity)
@@ -287,9 +406,9 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: str
         :return: None
         """
-        refinement_ = refinement.AbstractRefinement(**cfg[input_step])
-        logging.info('Subpixel refinement...')
-        if self.right_disp_map != 'accurate':
+        refinement_ = refinement.AbstractRefinement(**cfg[input_step])  # type: ignore
+        logging.info("Subpixel refinement...")
+        if self.right_disp_map != "accurate":
             refinement_.subpixel_refinement(self.left_cv, self.left_disparity)
         else:
             refinement_.subpixel_refinement(self.left_cv, self.left_disparity)
@@ -304,53 +423,55 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: str
         :return: None
         """
-        validation_ = validation.AbstractValidation(**cfg[input_step])
+        validation_ = validation.AbstractValidation(**cfg[input_step])  # type: ignore
 
-        logging.info('Validation...')
+        logging.info("Validation...")
 
-        if self.right_disp_map != 'accurate':
+        if self.right_disp_map != "accurate":
             self.left_disparity = validation_.disparity_checking(self.left_disparity, self.right_disparity)
 
         else:
             self.left_disparity = validation_.disparity_checking(self.left_disparity, self.right_disparity)
             self.right_disparity = validation_.disparity_checking(self.right_disparity, self.left_disparity)
             # Interpolated mismatch and occlusions
-            if 'interpolated_disparity' in cfg[input_step]:
-                interpolate_ = validation.AbstractInterpolation(**cfg[input_step])
+            if "interpolated_disparity" in cfg[input_step]:
+                interpolate_ = validation.AbstractInterpolation(**cfg[input_step])  # type: ignore
                 interpolate_.interpolated_disparity(self.left_disparity)
                 interpolate_.interpolated_disparity(self.right_disparity)
 
     def run_multiscale(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
-            Compute the disparity range for the next scale
-            :param cfg: pipeline configuration
-            :type  cfg: dict
-            :param input_step: step to trigger
-            :type input_step: str
-            :return: None
-            """
+        Compute the disparity range for the next scale
+        :param cfg: pipeline configuration
+        :type  cfg: dict
+        :param input_step: step to trigger
+        :type input_step: str
+        :return: None
+        """
 
-        logging.info('Disparity range computation...')
+        logging.info("Disparity range computation...")
 
-        multiscale_ = multiscale.AbstractMultiscale(**cfg[input_step])
+        multiscale_ = multiscale.AbstractMultiscale(**cfg[input_step])  # type: ignore
 
         # Update min and max user disparity according to the current scale
         self.dmin_user = self.dmin_user * self.scale_factor
         self.dmax_user = self.dmax_user * self.scale_factor
 
         # Compute disparity range for the next scale level
-        if self.right_disp_map != 'accurate':
-            self.disp_min, self.disp_max = multiscale_.disparity_range(self.left_disparity,
-                                                                        self.dmin_user, self.dmax_user)
+        if self.right_disp_map != "accurate":
+            self.disp_min, self.disp_max = multiscale_.disparity_range(
+                self.left_disparity, self.dmin_user, self.dmax_user
+            )
         else:
-            self.disp_min, self.disp_max = multiscale_.disparity_range(self.left_disparity,
-                                                                        self.dmin_user, self.dmax_user)
+            self.disp_min, self.disp_max = multiscale_.disparity_range(
+                self.left_disparity, self.dmin_user, self.dmax_user
+            )
             # Update min and max user disparity according to the current scale
             self.dmin_user_right = self.dmin_user_right * self.scale_factor
             self.dmax_user_right = self.dmax_user_right * self.scale_factor
-            self.right_disp_min, self.right_disp_max = multiscale_.disparity_range(self.right_disparity,
-                                                                                    self.dmin_user_right,
-                                                                                    self.dmax_user_right)
+            self.right_disp_min, self.right_disp_max = multiscale_.disparity_range(
+                self.right_disparity, self.dmin_user_right, self.dmax_user_right
+            )
 
         # Get the next scale's images
         self.left_img = self.img_left_pyramid.pop(0)
@@ -359,10 +480,43 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         # Update the current scale for the next state
         self.current_scale = self.current_scale - 1
 
-    def run_prepare(self, cfg: Dict[str, dict], left_img: xr.Dataset, right_img: xr.Dataset,
-                    disp_min: Union[np.array, int], disp_max: Union[np.array, int], scale_factor: int,
-                    num_scales: int, right_disp_min: Union[None, np.array] = None,
-                    right_disp_max: Union[None, np.array] = None) -> None:
+    def cost_volume_confidence_run(self, cfg: Dict[str, dict], input_step: str) -> None:
+        """
+        Confidence prediction
+        :param cfg: pipeline configuration
+        :type  cfg: dict
+        :param input_step: step to trigger
+        :type input_step: str
+        :return: None
+        """
+        confidence_ = cost_volume_confidence.AbstractCostVolumeConfidence(**cfg[input_step])  # type: ignore
+
+        logging.info("Confidence prediction...")
+
+        if self.right_disp_map == "none":
+            self.left_disparity, self.left_cv = confidence_.confidence_prediction(
+                self.left_disparity, self.left_img, self.right_img, self.left_cv
+            )
+        else:
+            self.left_disparity, self.left_cv = confidence_.confidence_prediction(
+                self.left_disparity, self.left_img, self.right_img, self.left_cv
+            )
+            self.right_disparity, self.right_cv = confidence_.confidence_prediction(
+                self.right_disparity, self.right_img, self.left_img, self.right_cv
+            )
+
+    def run_prepare(
+        self,
+        cfg: Dict[str, dict],
+        left_img: xr.Dataset,
+        right_img: xr.Dataset,
+        disp_min: Union[np.array, int],
+        disp_max: Union[np.array, int],
+        scale_factor: Union[None, int] = None,
+        num_scales: Union[None, int] = None,
+        right_disp_min: Union[None, np.array] = None,
+        right_disp_max: Union[None, np.array] = None,
+    ) -> None:
         """
         Prepare the machine before running
 
@@ -383,25 +537,28 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :param disp_max: maximal disparity
         :type disp_max: int or np.array
         :param scale_factor: scale factor for multiscale
-        :type scale_factor: int
+        :type scale_factor: int or None
         :param num_scales: scales number for multiscale
-        :type num_scales: int
+        :type num_scales: int or None
         :param disp_min_right: minimal disparity of the right image
         :type disp_min_right: np.array or None
         :param disp_max_right: maximal disparity of the right image
         :type disp_max_right: np.array or None
         :return: None
         """
-        # Total number of scales
-        self.num_scales = num_scales
-        # Scale factor
-        self.scale_factor = scale_factor
-
+        # Mono-resolution processing by default if num_scales or scale_factor are not specified
+        if num_scales is None or scale_factor is None:
+            self.num_scales = 1
+            self.scale_factor = 1
+        else:
+            self.num_scales = num_scales
+            self.scale_factor = scale_factor
 
         if self.num_scales > 1:
             # If multiscale processing, create pyramid and select first scale's images
-            self.img_left_pyramid, self.img_right_pyramid = prepare_pyramid(left_img, right_img, self.num_scales,
-                                                                            scale_factor)
+            self.img_left_pyramid, self.img_right_pyramid = prepare_pyramid(
+                left_img, right_img, self.num_scales, scale_factor
+            )
             self.left_img = self.img_left_pyramid.pop(0)
             self.right_img = self.img_right_pyramid.pop(0)
             # Initialize current scale
@@ -439,12 +596,11 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
                 self.right_disp_min = -self.disp_max
                 self.right_disp_max = -self.disp_min
 
-
         # Initiate output disparity datasets
         self.left_disparity = xr.Dataset()
         self.right_disparity = xr.Dataset()
         # To determine whether the right disparity map has to be computed
-        self.right_disp_map = cfg['right_disp_map']['method']
+        self.right_disp_map = cfg["right_disp_map"]["method"]
         # Add transitions
         self.add_transitions(self._transitions_run)
 
@@ -470,13 +626,14 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
             # But there's only a filter transition. Therefore, in the case of filter.1 we have to call the
             # filter
             # trigger and give the configuration of filter.1
-            if len(input_step.split('.')) != 1:
-                self.trigger(input_step.split('.')[0], cfg, input_step)
+            if len(input_step.split(".")) != 1:
+                self.trigger(input_step.split(".")[0], cfg, input_step)
             else:
                 self.trigger(input_step, cfg, input_step)
         except (MachineError, KeyError):
-            print('\n A problem occurs during Pandora running ' + input_step +
-                  '. Be sure of your sequencement step  \n')
+            print(
+                "\n A problem occurs during Pandora running " + input_step + ". Be sure of your sequencement step  \n"
+            )
             raise
 
     def run_exit(self) -> None:
@@ -486,10 +643,11 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :return: None
         """
         self.remove_transitions(self._transitions_run)
-        self.set_state('begin')
+        self.set_state("begin")
 
-    def right_disp_map_check_conf(self, cfg: Dict[str, dict],
-                                  input_step: str) -> None:  # pylint:disable=unused-argument
+    def right_disp_map_check_conf(
+        self, cfg: Dict[str, dict], input_step: str  # pylint:disable=unused-argument
+    ) -> None:
         """
         Check the right_disp_map configuration
 
@@ -500,15 +658,13 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :return: None
         """
 
-        schema = {
-            'method': And(str, lambda input: 'none' or 'accurate')
-        }
+        schema = {"method": And(str, lambda input: "none" or "accurate")}
 
         checker = Checker(schema)
-        checker.validate(cfg['right_disp_map'])
+        checker.validate(cfg["right_disp_map"])
 
         # Store the righ disparity map configuration
-        self.right_disp_map = cfg['right_disp_map']['method']
+        self.right_disp_map = cfg["right_disp_map"]["method"]
 
     def matching_cost_check_conf(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -521,8 +677,8 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :return: None
         """
 
-        matching_cost_ = matching_cost.AbstractMatchingCost(**cfg[input_step])
-        self.pipeline_cfg['pipeline'][input_step] = matching_cost_.cfg
+        matching_cost_ = matching_cost.AbstractMatchingCost(**cfg[input_step])  # type: ignore
+        self.pipeline_cfg["pipeline"][input_step] = matching_cost_.cfg
 
     def disparity_check_conf(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -534,8 +690,8 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: string
         :return: None
         """
-        disparity_ = disparity.AbstractDisparity(**cfg[input_step])
-        self.pipeline_cfg['pipeline'][input_step] = disparity_.cfg
+        disparity_ = disparity.AbstractDisparity(**cfg[input_step])  # type: ignore
+        self.pipeline_cfg["pipeline"][input_step] = disparity_.cfg
 
     def filter_check_conf(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -547,8 +703,8 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: string
         :return: None
         """
-        filter_ = filter.AbstractFilter(**cfg[input_step])
-        self.pipeline_cfg['pipeline'][input_step] = filter_.cfg
+        filter_ = filter.AbstractFilter(**cfg[input_step])  # type: ignore
+        self.pipeline_cfg["pipeline"][input_step] = filter_.cfg
 
     def refinement_check_conf(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -560,8 +716,8 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: string
         :return: None
         """
-        refinement_ = refinement.AbstractRefinement(**cfg[input_step])
-        self.pipeline_cfg['pipeline'][input_step] = refinement_.cfg
+        refinement_ = refinement.AbstractRefinement(**cfg[input_step])  # type: ignore
+        self.pipeline_cfg["pipeline"][input_step] = refinement_.cfg
 
     def aggregation_check_conf(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -573,8 +729,8 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: string
         :return: None
         """
-        aggregation_ = aggregation.AbstractAggregation(**cfg[input_step])
-        self.pipeline_cfg['pipeline'][input_step] = aggregation_.cfg
+        aggregation_ = aggregation.AbstractAggregation(**cfg[input_step])  # type: ignore
+        self.pipeline_cfg["pipeline"][input_step] = aggregation_.cfg
 
     def optimization_check_conf(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -586,8 +742,8 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: string
         :return: None
         """
-        optimization_ = optimization.AbstractOptimization(**cfg[input_step])
-        self.pipeline_cfg['pipeline'][input_step] = optimization_.cfg
+        optimization_ = optimization.AbstractOptimization(**cfg[input_step])  # type: ignore
+        self.pipeline_cfg["pipeline"][input_step] = optimization_.cfg
 
     def validation_check_conf(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
@@ -600,16 +756,18 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :return: None
         """
 
-        validation_ = validation.AbstractValidation(**cfg[input_step])
-        self.pipeline_cfg['pipeline'][input_step] = validation_.cfg
-        if 'interpolated_disparity' in validation_.cfg:
-            interpolate_ = validation.AbstractInterpolation(**cfg[input_step])  # pylint: disable=unused-variable
+        validation_ = validation.AbstractValidation(**cfg[input_step])  # type: ignore
+        self.pipeline_cfg["pipeline"][input_step] = validation_.cfg
+        if "interpolated_disparity" in validation_.cfg:
+            interpolate_ = validation.AbstractInterpolation(**cfg[input_step])  # type: ignore # pylint: disable=unused-variable
 
-        if validation_.cfg['validation_method'] == 'cross_checking' and self.right_disp_map != 'accurate':
-            raise MachineError('Can t trigger event cross-checking validation  if right_disp_map method equal to '
-                               + self.right_disp_map)
+        if validation_.cfg["validation_method"] == "cross_checking" and self.right_disp_map != "accurate":
+            raise MachineError(
+                "Can t trigger event cross-checking validation  if right_disp_map method equal to "
+                + self.right_disp_map
+            )
 
-    def multiscale_check_conf(self, cfg: Dict[str, dict], input_step: str):
+    def multiscale_check_conf(self, cfg: Dict[str, dict], input_step: str) -> None:
         """
         Check the disparity computation configuration
 
@@ -619,10 +777,23 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         :type input_step: string
         :return:
         """
-        multiscale_ = multiscale.AbstractMultiscale(**cfg[input_step])
-        self.pipeline_cfg['pipeline'][input_step] = multiscale_.cfg
+        multiscale_ = multiscale.AbstractMultiscale(**cfg[input_step])  # type: ignore
+        self.pipeline_cfg["pipeline"][input_step] = multiscale_.cfg
 
-    def check_conf(self, cfg: Dict[str, dict]):
+    def cost_volume_confidence_check_conf(self, cfg: Dict[str, dict], input_step: str) -> None:
+        """
+        Check the confidence configuration
+
+        :param cfg: configuration
+        :type cfg: dict
+        :param input_step: current step
+        :type input_step: string
+        :return: None
+        """
+        confidence_ = cost_volume_confidence.AbstractCostVolumeConfidence(**cfg[input_step])  # type: ignore
+        self.pipeline_cfg["pipeline"][input_step] = confidence_.cfg
+
+    def check_conf(self, cfg: Dict[str, dict]) -> None:
         """
         Check configuration and transitions
 
@@ -649,23 +820,25 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
                 # filter
                 # trigger and give the configuration of filter.1
 
-                if len(input_step.split('.')) != 1:
-                    self.trigger(input_step.split('.')[0], cfg, input_step)
+                if len(input_step.split(".")) != 1:
+                    self.trigger(input_step.split(".")[0], cfg, input_step)
                 else:
                     self.trigger(input_step, cfg, input_step)
 
             except (MachineError, KeyError):
-                print('\n Problem during Pandora checking configuration steps sequencing. '
-                      'Check your configuration file. \n')
+                print(
+                    "\n Problem during Pandora checking configuration steps sequencing. "
+                    "Check your configuration file. \n"
+                )
                 raise
 
         # Remove transitions
         self.remove_transitions(self._transitions_check)
 
         # Coming back to the initial state
-        self.set_state('begin')
+        self.set_state("begin")
 
-    def remove_transitions(self, transition_list: Dict[str, dict]) -> None:
+    def remove_transitions(self, transition_list: List[Dict[str, str]]) -> None:
         """
         Delete all transitions defined in the input list
 
@@ -678,11 +851,11 @@ class PandoraMachine(Machine):  # pylint:disable=too-many-instance-attributes
         # deleted_triggers list is used to avoid multiple call of 'remove_transition'' with the same trigger name.
         deleted_triggers = []
         for trans in transition_list:
-            if trans['trigger'] not in deleted_triggers:
-                self.remove_transition(trans['trigger'])
-                deleted_triggers.append(trans['trigger'])
+            if trans not in deleted_triggers:
+                self.remove_transition(trans["trigger"])
+                deleted_triggers.append(trans["trigger"])
 
-    def is_not_last_scale(self, input_step: str, cfg: Dict[str, dict]):  # pylint:disable=unused-argument
+    def is_not_last_scale(self, input_step: str, cfg: Dict[str, dict]) -> bool:  # pylint:disable=unused-argument
         """
         Check if the current scale is the last scale
         :param cfg: configuration
